﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace WpfPdfReader
{
    public class ImageHelper
    {
        public static byte[] ImageToBytes(Image image)
        {
            if (image == null)
                return null;
            using (MemoryStream ms = new MemoryStream())
            {
                using (Bitmap bitmap = new Bitmap(image))
                {
                    try
                    {
                        bitmap.Save(ms, image.RawFormat);
                    }
                    catch
                    {
                        bitmap.Save(ms, System.Drawing.Imaging.ImageFormat.Png);
                    }
                    return ms.ToArray();
                }
            }
        }

        public static Image ReadImage(byte[] bytes)
        {
            if (bytes == null)
                return null;
            //如果用using 会引起 "A generic error occurred in GDI+"
            MemoryStream memStream = new MemoryStream(bytes);
            {
                Image img = Image.FromStream(memStream);
                //img.Save(@"test2.jpg");
                return img;
            }
        }

        /// <summary>
        /// 保持纵横比
        /// </summary>
        /// <param name="srcImage"></param>
        /// <param name="newSize"></param>
        /// <returns></returns>
        public static Image GetThumbnailImage(Image srcImage, Size newSize)
        {
            if (srcImage != null && newSize != null)
            {
                int srcWidth = srcImage.Width;
                int srcHeight = srcImage.Height;
                int dstHeight = newSize.Height;
                int dstWidth = newSize.Width;
                if (srcWidth > srcHeight)
                {
                    dstHeight = newSize.Height;
                    dstWidth = Convert.ToInt32(dstHeight * (new decimal(srcWidth) / new decimal(srcHeight)));
                }
                else
                {
                    dstWidth = newSize.Width;
                    dstHeight = Convert.ToInt32(dstWidth * (new decimal(srcHeight) / new decimal(srcWidth)));
                }

                return GetImage(srcImage, new Size(dstWidth, dstHeight));
            }
            else
                return null;
        }

        /// <summary>
        /// 根据是否需要保持纵横比返回缩略图
        /// </summary>
        /// <param name="srcImage">原始图像</param>
        /// <param name="newSize">新图像大小</param>
        /// <param name="keepAspectRatio">是否保持纵横比</param>
        /// <returns></returns>
        public static Image GetThumbnailImage(Image srcImage, Size newSize, bool keepAspectRatio)
        {
            if (keepAspectRatio)
                return GetThumbnailImage(srcImage, newSize);
            else
            {
                return GetImage(srcImage, newSize);
            }
        }

        public static Icon ImageToIcon(Image image, string fileName)
        {
            Bitmap bmp = image as Bitmap;
            Icon icon = System.Drawing.Icon.FromHandle(bmp.GetHicon());
            FileStream fs = new FileStream(fileName, FileMode.OpenOrCreate);
            icon.Save(fs);
            fs.Close();
            return icon;
        }

        private static Image GetImage(Image srcImage, Size newSize)
        {
            int srcWidth = srcImage.Width;
            int srcHeight = srcImage.Height;
            int dstHeight = newSize.Height;
            int dstWidth = newSize.Width;

            Image dstImage = new Bitmap(dstWidth, dstHeight);
            Graphics g = Graphics.FromImage(dstImage);
            g.SmoothingMode = SmoothingMode.HighQuality;
            g.CompositingQuality = CompositingQuality.HighQuality;
            g.InterpolationMode = InterpolationMode.High;
            Rectangle dstRect = new Rectangle(0, 0, dstWidth, dstHeight);
            Rectangle srcRect = new Rectangle(0, 0, srcWidth, srcHeight);

            Image newImage = srcImage.Clone() as Image;
            g.DrawImage(newImage, dstRect, srcRect, GraphicsUnit.Pixel);

            using (MemoryStream ms = new MemoryStream())
            {
                try
                {
                    dstImage.Save(ms, System.Drawing.Imaging.ImageFormat.Png);
                    dstImage = Image.FromStream(ms);
                }
                catch { }
            }

            g.Dispose();

            return dstImage;
        }
    }
}
