﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WpfPdfReader
{
    /// <summary>
    /// LabPdfReaderControl.xaml 的交互逻辑
    /// </summary>
    public partial class LabPdfReaderControl : UserControl
    {
        private MupdfSharp.PdfRenderRealtime pdfrender = null;
        public event EventHandler<OpenCompletedEventArgs> OpenCompleted;
        public event EventHandler<ViewTypeChangedEventArgs> ViewTypeChanged;

        public LabPdfReaderControl()
        {
            InitializeComponent();
            this.Loaded += new RoutedEventHandler(LabPdfReaderControl_Loaded);
            this.OpenCompleted += new EventHandler<OpenCompletedEventArgs>(LabPdfReaderControl_OpenCompleted);
            this.ViewTypeChanged += new EventHandler<ViewTypeChangedEventArgs>(LabPdfReaderControl_ViewTypeChanged);
            AppDomain.CurrentDomain.ProcessExit += new EventHandler(CurrentDomain_ProcessExit);
            recentItemControl.ItemSelectionChanged += new EventHandler<RecentItemControl.FileItemSelectionChangedEventArgs>(recentItemControl_ItemSelectionChanged);
            recentItemControl.Open += new RoutedEventHandler(recentItemControl_Open);
        }

        void OnViewTypeChanged(ViewTypeChangedEventArgs e)
        {
            if (this.ViewTypeChanged != null)
            {
                this.ViewTypeChanged(this, e);
            }
        }

        void LabPdfReaderControl_ViewTypeChanged(object sender, ViewTypeChangedEventArgs e)
        {
            if (e.View == ViewType.ReadView)
            {
                imageBorder.Child = MasterScrollViewer;

                this.btnPrevious.IsEnabled = true;
                this.btnNext.IsEnabled = true;
                this.btnZoomIn.IsEnabled = true;
                this.btnZoomOut.IsEnabled = true;
                this.textPage.IsEnabled = true;
            }
            else if (e.View == ViewType.Catalog)
            {
                imageBorder.Child = recentItemControl;

                this.btnPrevious.IsEnabled = false;
                this.btnNext.IsEnabled = false;
                this.btnZoomIn.IsEnabled = false;
                this.btnZoomOut.IsEnabled = false;
                this.textPage.IsEnabled = false;
            }
        }

        void LabPdfReaderControl_OpenCompleted(object sender, OpenCompletedEventArgs e)
        {
            this.btnGoBack.IsEnabled = true;
            this.EnableGoBack = true;
            this.IsGoBack = false;

            bool IsAdded = false;
            foreach (MenuItem item in this.contextMenu.Items)
            {
                if (item.Header.ToString() == FileName)
                {
                    IsAdded = true;
                    break;
                }
            }
            if (!IsAdded)
            {
                MenuItem newItem = new MenuItem() { Header = FileName };
                newItem.Icon = new Image() { Width = 16, Height = 16, Source = BrushHelper.GetBitmapImage(new Uri("images/PushPin.png", UriKind.Relative)) };

                newItem.Click += new RoutedEventHandler((menuItem, args) =>
                {
                    string fileName = (menuItem as MenuItem).Header.ToString();
                    Open(fileName);
                });

                this.contextMenu.Items.Add(newItem);
            }

            this.recentItemControl.AddFileItem(FileName, this.pdfrender.CurrentPageBitmap);

            this.View = ViewType.ReadView;
        }

        void CurrentDomain_ProcessExit(object sender, EventArgs e)
        {
            if (this.pdfrender != null)
            {
                this.recentItemControl.AddFileItem(this.FileName, this.pdfrender.CurrentPage, this.pdfrender.CurrentPageBitmap);
                this.recentItemControl.Serialize();
                this.pdfrender.Close();
            }
        }

        RecentItemControl recentItemControl = new RecentItemControl();
        void LabPdfReaderControl_Loaded(object sender, RoutedEventArgs e)
        {
            if (recentItemControl.HasItem && string.IsNullOrEmpty(this.FileName))
            {
                this.View = ViewType.Catalog;
            }
        }

        void recentItemControl_Open(object sender, RoutedEventArgs e)
        {
            this.OpenPdf_Click(sender, e);
        }

        void recentItemControl_ItemSelectionChanged(object sender, RecentItemControl.FileItemSelectionChangedEventArgs e)
        {
            string fileName = e.Item.FullName;
            if (System.IO.File.Exists(fileName))
            {
                Open(fileName);
            }
            else
            {
                MessageBox.Show(fileName + "\n不存在", "提示", MessageBoxButton.OK, MessageBoxImage.Question);
            }
        }

        public bool Open(string fileName)
        {
            if (this.FileName == fileName && (!string.IsNullOrEmpty(fileName)))
            {
                this.View = ViewType.ReadView;
                if (OpenCompleted != null)
                    OpenCompleted(this, new OpenCompletedEventArgs(FileName));
                return true;
            }
            else if (System.IO.File.Exists(fileName))
            {
                this.HWPercent = 0.0f;

                if (this.pdfrender != null)
                {
                    this.recentItemControl.AddFileItem(this.FileName, this.pdfrender.CurrentPage, this.pdfrender.CurrentPageBitmap);
                }
                pdfrender = new MupdfSharp.PdfRenderRealtime();
                try
                {
                    if (pdfrender.OpenFile(fileName, new System.Drawing.SizeF((float)this.MasterScrollViewer.ActualWidth, (float)(this.MasterScrollViewer.ActualHeight))))
                    {
                        var fileItem = this.recentItemControl.FindItem(fileName);
                        if (fileItem != null && fileItem.Page > 0)
                        {
                            WindowPrompt win = new WindowPrompt(fileItem);
                            if (win.ShowDialog().Value)
                            {
                                GoToPage(fileItem.Page + 1);
                            }
                            else
                            {
                                GotoFirstPage();
                            }
                            //MessageBoxResult msgResult = MessageBox.Show(string.Format("您上次浏览到第{0}页,是否继续浏览?", fileItem.Page + 1), "提示", MessageBoxButton.YesNo, MessageBoxImage.Information);
                            //if (msgResult == MessageBoxResult.Yes)
                            //{
                            //    GoToPage(fileItem.Page + 1);
                            //}
                            //else
                            //{
                            //    GotoFirstPage();
                            //}
                        }
                        else
                        {
                            GotoFirstPage();
                        }

                        this.FileName = fileName;

                        if (OpenCompleted != null)
                            OpenCompleted(this, new OpenCompletedEventArgs(FileName));

                        return true;
                    }
                    else
                    {
                        pdfrender.Close();
                        pdfrender = null;
                        return false;
                    }
                }
                catch
                {
                    pdfrender.Close();
                    pdfrender = null;
                    return false;
                }
            }
            else
                return false;
        }

        private string fileName = "";
        public string FileName
        {
            set
            {
                if (value != fileName)
                    fileName = value;
            }
            get
            {
                return fileName;
            }
        }

        private ViewType viewType = ViewType.None;
        public ViewType View
        {
            get { return this.viewType; }
            set
            {
                if (value != this.viewType)
                {
                    this.viewType = value;
                    OnViewTypeChanged(new ViewTypeChangedEventArgs(value));
                }
            }
        }

        public void GotoFirstPage()
        {
            if (this.pdfrender != null)
            {
                this.pdfrender.GotoFirstPage();
                SetImage();
            }
        }

        private void SetImage()
        {
            if (this.HWPercent == 0)
            {
                this.HWPercent = this.pdfrender.CurrentScale;
                this.currentPercent = this.HWPercent;
            }
            this.image.Source = ImageSourceHelper.ImageToSource(this.pdfrender.CurrentPageBitmap);
            this.textPage.Text = string.Format("{0}/{1}", this.pdfrender.CurrentPage + 1, this.pdfrender.PageCount);
            this.MasterScrollViewer.ScrollToTop();
        }

        public void GotoLastPage()
        {
            if (this.pdfrender != null)
            {
                this.pdfrender.GotoLastPage();
                SetImage();
            }
        }

        public void GotoNextPage()
        {
            if (this.pdfrender != null)
            {
                this.pdfrender.GotoNextPage();
                SetImage();
            }
        }

        public void GotoPreviousPage()
        {
            if (this.pdfrender != null)
            {
                this.pdfrender.GotoPreviousPage();
                SetImage();
            }
        }

        public void GoToPage(int page)
        {
            if (this.pdfrender != null)
            {
                this.pdfrender.SetCurrentPage(page - 1);
                SetImage();
            }
        }

        private void btnFirst_MouseDown(object sender, MouseButtonEventArgs e)
        {
            GotoFirstPage();
        }

        /// <summary>
        /// 显示第一张页面时长宽缩放比例
        /// </summary>
        private float HWPercent = 0.0f;
        /// <summary>
        /// 当前页面长宽缩放比例
        /// </summary>
        private float currentPercent = 0.0f;

        private void btnPrevious_MouseUp(object sender, MouseButtonEventArgs e)
        {
            GotoPreviousPage();
        }

        private void btnNext_MouseUp(object sender, MouseButtonEventArgs e)
        {
            GotoNextPage();
        }

        private bool EnableGoBack = false;
        private bool IsGoBack = false;

        public void GoBack()
        {
            if (this.EnableGoBack)
            {
                if (!IsGoBack)
                {
                    IsGoBack = true;
                    this.View = ViewType.Catalog;
                }
                else
                {
                    IsGoBack = false;
                    this.View = ViewType.ReadView;
                }
            }
        }

        private void btnGoBack_MouseUp(object sender, MouseButtonEventArgs e)
        {
            this.GoBack();
        }

        #region 通过鼠标进行 缩放、平移图片

        private bool m_IsMouseLeftButtonDown;
        private Point m_PreviousMousePoint;

        private void MasterImage_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            //var rectangle = sender as ContentControl;
            //if (rectangle == null) return;
            //rectangle.CaptureMouse();
            //m_IsMouseLeftButtonDown = true;
            //m_PreviousMousePoint = e.GetPosition(rectangle);
        }

        private void MasterImage_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            //var rectangle = sender as ContentControl;
            //if (rectangle == null) return;
            //rectangle.ReleaseMouseCapture();
            //m_IsMouseLeftButtonDown = false;
        }

        private void MasterImage_MouseMove(object sender, MouseEventArgs e)
        {
            //var rectangle = sender as ContentControl;
            //if (rectangle == null) return;
            //if (m_IsMouseLeftButtonDown)
            //    DoImageMove(rectangle, e);
        }

        private void DoImageMove(ContentControl rectangle, MouseEventArgs e)
        {
            if (e.LeftButton != MouseButtonState.Pressed) return;
            var group = imageBorder.FindResource("ImageTransformResource") as TransformGroup;
            var transform = group.Children[1] as TranslateTransform;
            var position = e.GetPosition(rectangle);
            transform.X += position.X - m_PreviousMousePoint.X;
            transform.Y += position.Y - m_PreviousMousePoint.Y;
            m_PreviousMousePoint = position;
        }

        private void MasterImage_MouseWheel(object sender, MouseWheelEventArgs e)
        {
            //var image = sender as ContentControl;
            //if (image == null) return;
            //var group = imageBorder.FindResource("ImageTransformResource") as TransformGroup;
            //var point = e.GetPosition(image);
            //var scale = e.Delta * 0.001;
            //currentPercent += (float)scale;
            //if (currentPercent < this.HWPercent)
            //    currentPercent = this.HWPercent;
            //this.pdfrender.ScalePage(currentPercent);
            //SetImage();
            // ZoomImage(group, point, scale);
        }

        #endregion


        private void btnZoomIn_MouseUp(object sender, MouseButtonEventArgs e)
        {
            ZoomIn();
        }

        #region 多点触摸操作

        private void image_ManipulationStarting(object sender, ManipulationStartingEventArgs e)
        {
            e.ManipulationContainer = touchPad;
            e.Mode = ManipulationModes.All;
        }

        private void image_ManipulationDelta(object sender, ManipulationDeltaEventArgs e)
        {
            FrameworkElement element = (FrameworkElement)e.Source;
            element.Opacity = 0.6;

            Matrix matrix = (((TransformGroup)element.RenderTransform).Children[2] as MatrixTransform).Matrix;

            var deltaManipulation = e.DeltaManipulation;

            Point center = new Point(element.ActualWidth / 2, element.ActualHeight / 2);
            center = matrix.Transform(center);

            this.pdfrender.ScalePage((float)(deltaManipulation.Scale.X + deltaManipulation.Scale.Y) / 2);
            SetImage();

            matrix.ScaleAt(deltaManipulation.Scale.X, deltaManipulation.Scale.Y, center.X, center.Y);

            matrix.RotateAt(e.DeltaManipulation.Rotation, center.X, center.Y);

            matrix.Translate(e.DeltaManipulation.Translation.X, e.DeltaManipulation.Translation.Y);

            ((MatrixTransform)element.RenderTransform).Matrix = matrix;
        }

        private void image_ManipulationCompleted(object sender, ManipulationCompletedEventArgs e)
        {
            FrameworkElement element = (FrameworkElement)e.Source;
            element.Opacity = 1;
        }

        /// <summary>
        /// 惯性操作
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void image_ManipulationInertiaStarting(object sender, ManipulationInertiaStartingEventArgs e)
        {
            e.TranslationBehavior = new InertiaTranslationBehavior();
            e.TranslationBehavior.InitialVelocity = e.InitialVelocities.LinearVelocity;
            e.TranslationBehavior.DesiredDeceleration = 10.0 * 96.0 / (1000.0 * 1000.0);

            e.ExpansionBehavior = new InertiaExpansionBehavior();
            e.ExpansionBehavior.InitialVelocity = e.InitialVelocities.ExpansionVelocity;
            e.ExpansionBehavior.DesiredDeceleration = 0.1 * 96 / 1000.0 * 1000.0;

            e.RotationBehavior = new InertiaRotationBehavior();
            e.RotationBehavior.InitialVelocity = e.InitialVelocities.AngularVelocity;
            e.RotationBehavior.DesiredDeceleration = 720 / (1000.0 * 1000.0);
        }

        #endregion

        private void btnZoomOut_MouseUp(object sender, MouseButtonEventArgs e)
        {
            ZoomOut();
        }
        /// <summary>
        /// 以25%的比例放大
        /// </summary>
        public void ZoomIn()
        {
            currentPercent += currentPercent * 0.25f;
            this.pdfrender.ScalePage(currentPercent);
            SetImage();
        }
        /// <summary>
        /// 以25%的比例缩小
        /// </summary>
        public void ZoomOut()
        {
            currentPercent -= currentPercent * 0.25f;
            if (currentPercent < HWPercent)
                currentPercent = HWPercent;
            this.pdfrender.ScalePage(currentPercent);
            SetImage();
        }

        private void OpenPdf_Click(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog openDlg = new Microsoft.Win32.OpenFileDialog();
            openDlg.Filter = "PDF File(*.pdf)|*.pdf";
            if (openDlg.ShowDialog(Owner).Value)
            {
                Open(openDlg.FileName);
            }
        }

        private static Window owner = null;
        private static Window Owner
        {
            get
            {
                WindowCollection wins = Application.Current.Windows;
                foreach (Window win in wins)
                {
                    if (win.IsActive)
                    {
                        owner = win;
                        break;
                    }
                }
                return owner;
            }
        }

        #region CommandBindings 事件处理

        private void ZoomIn_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            this.ZoomIn();
        }

        private void ZoomOut_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            this.ZoomOut();
        }

        private void CZoomIn_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = true;
        }

        private void CZoomOut_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = true;
        }

        #endregion

        private void UserControl_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            //if (e.Key == Key.PageDown || e.Key == Key.Right || e.Key == Key.L || e.Key == Key.K  || e.Key == Key.S)
            //    this.GotoNextPage();
            //else if (e.Key == Key.PageUp || e.Key == Key.Left || e.Key == Key.J || e.Key == Key.I  || e.Key == Key.A)
            //    this.GotoPreviousPage();
            //else if (e.Key == Key.Home)
            //    this.GotoFirstPage();
            //else if (e.Key == Key.End)
            //    this.GotoLastPage();
            //else if (e.Key == Key.F)
            //    this.ZoomIn();
            //else if (e.Key == Key.D)
            //    this.ZoomOut();

        }

        public void ScrollToHome()
        {
            double offset = this.MasterScrollViewer.VerticalOffset - 20;
            this.MasterScrollViewer.ScrollToVerticalOffset(offset);
        }
        public void ScrollToBottom()
        {
            double offset = this.MasterScrollViewer.VerticalOffset + 20;
            this.MasterScrollViewer.ScrollToVerticalOffset(offset);
        }
        public void ScrollToRight()
        {
            double offset = this.MasterScrollViewer.HorizontalOffset + 10;
            this.MasterScrollViewer.ScrollToHorizontalOffset(offset);
        }
        public void ScrollToLeft()
        {
            double offset = this.MasterScrollViewer.HorizontalOffset - 10;
            this.MasterScrollViewer.ScrollToHorizontalOffset(offset);
        }

        public void AddBookMark()
        {
            WindowAddBookmark win = new WindowAddBookmark();
            if (win.ShowDialog().Value)
            {
                this.recentItemControl.AddBookMark(this.FileName, win.BookMarkName, this.pdfrender.CurrentPage, this.pdfrender.CurrentPageBitmap, win.Note);
            }
        }

        public void ShowBookManager()
        {
            WindowBookManager win = new WindowBookManager(this.recentItemControl.GetBookMarks(this.FileName));
            win.GoToBookMark += new EventHandler<BookMarkEventArgs>(win_GoToBookMark);
            win.ShowDialog();
        }

        void win_GoToBookMark(object sender, BookMarkEventArgs e)
        {
            this.GoToPage(e.Page + 1);
        }

        private void textPage_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (this.pdfrender != null)
            {
                if (e.Key == Key.Return)
                {
                    int page;
                    int.TryParse(this.textPage.Text, out page);
                    this.GoToPage(page);
                    this.Focus();
                }
            }
        }

        private void UserControl_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            if (this.pdfrender != null)
            {
                if (this.pdfrender.CurrentPageBitmap.Width > e.NewSize.Width || this.pdfrender.CurrentPageBitmap.Height > e.NewSize.Height)
                    return;
                this.pdfrender.SetPageSize(new System.Drawing.SizeF((float)e.NewSize.Width, (float)e.NewSize.Height));
                this.pdfrender.SetCurrentPage(this.pdfrender.CurrentPage);
                this.SetImage();
                this.HWPercent = this.pdfrender.CurrentScale;
            }
        }

        private void UserControl_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
                if (files != null && files.Length > 0)
                {
                    string fileName = files[0];
                    if (System.IO.Path.GetExtension(fileName).ToUpper() == ".PDF")
                    {
                        e.Effects = DragDropEffects.All;
                    }
                    else
                        e.Effects = DragDropEffects.None;
                }
                else
                    e.Effects = DragDropEffects.None;
            }
        }

        private void UserControl_Drop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
                if (files != null && files.Length > 0)
                {
                    string fileName = files[0];
                    if (System.IO.Path.GetExtension(fileName).ToUpper() == ".PDF")
                    {
                        this.Open(fileName);
                    }
                }
            }
        }

        private void CExportImage_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            string pictureFolder = Environment.GetFolderPath(Environment.SpecialFolder.MyPictures);
            string appFolder = System.IO.Path.Combine(pictureFolder, "WpfPdfReader");
            if (!System.IO.Directory.Exists(appFolder))
                System.IO.Directory.CreateDirectory(appFolder);

            string fileFolder = System.IO.Path.Combine(appFolder, System.IO.Path.GetFileNameWithoutExtension(this.FileName));
            if (!System.IO.Directory.Exists(fileFolder))
                System.IO.Directory.CreateDirectory(fileFolder);

            string imgName = System.IO.Path.Combine(fileFolder, string.Format("{0}.png", this.pdfrender.CurrentPage + 1));
            this.pdfrender.CurrentPageBitmap.Save(imgName);
            MessageBox.Show("图片保存在\n" + imgName + "\n", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
        }

        private void CExportImage_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            if (this.View == ViewType.ReadView)
                e.CanExecute = this.pdfrender.CurrentPageBitmap != null;
            else
                e.CanExecute = false;
        }
    }

    public enum ViewType
    {
        /// <summary>
        /// 未知
        /// </summary>
        None = -1,
        /// <summary>
        /// 阅读模式
        /// </summary>
        ReadView = 0,
        /// <summary>
        /// 列表目录
        /// </summary>
        Catalog = 1,
    }

    public class OpenCompletedEventArgs : EventArgs
    {
        public string FileName { get; set; }
        public OpenCompletedEventArgs(string fileName)
        {
            this.FileName = fileName;
        }
    }

    public class ViewTypeChangedEventArgs : EventArgs
    {
        public ViewType View { get; set; }
        public ViewTypeChangedEventArgs(ViewType viewType)
        {
            this.View = viewType;
        }
    }
}
